﻿using IRaCIS.Core.Domain.Models;
using IRaCIS.Core.Domain.Share;
using IRaCIS.Core.Domain.Share.Reading;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace IRaCIS.Core.Infra.EFCore.Common
{
    public static class ReadingCommon
    {
        /// <summary>
        /// 获取访视的状态
        /// </summary>
        /// <param name="visit"></param>
        /// <returns></returns>
      
        public static ReadModuleEnum GetVisitSubmitStateEnum(SubjectVisit visit)
        {
            if (visit.SubmitState != SubmitStateEnum.Submitted)  // 未提交
            {
                return ReadModuleEnum.ImageUpload;
            }
            else if (visit.AuditState != AuditStateEnum.QCPassed) // 审核未通过
            {
                return ReadModuleEnum.ImageQuality;
            }
            else if (visit.ForwardState != ForwardStateEnum.Forwarded) // 未通过一致性核查
            {
                return ReadModuleEnum.ConsistencyCheck;
            }
            else
            {
                // 暂时
                return ReadModuleEnum.ReadComplete;
            }
        }


        public static bool IsNullOrEmpty(this string value)
        {
            if (value == null || value == string.Empty)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

     

        /// <summary>
        /// 获取DisplayName
        /// </summary>
        /// <param name="enumName"></param>
        /// <returns></returns>
        public static string GetDisplayName(this Enum enumName)
        {
            var type = enumName.GetType();//先获取这个枚举的类型
            var field = type.GetField(enumName.ToString());//通过这个类型获取到值
            var obj = (DisplayAttribute)field.GetCustomAttribute(typeof(DisplayAttribute));//得到特性
            return obj.Name ?? "";
        }
    }
}
